unit WinPlace;

(*
================================================================================
TWinPlace Component version 1.0 by Philip R. Henningsen - Public Domain - 6-May-96

           This is a modification of the TPlacement component ... just because
           I thought that the original version was too "verbose".  All credit
           should go to the original author.

           "TPlacement Component version 1.0 by Barry L. Harkness - Public Domain

           Feel free to use this component in any application, in
           any context, without any warranty.  This component is
           donated to the public domain to help fellow Delphi
           programmers dominate windows software development."

What's up with this?

          The TWinPlace component helps to automate the saving and restoring
          of TForm positions and WindowState properties.  You can drop the
          TWinPlace component onto any form, and specify to which .INI file,
          Section, and Identifier the TWinPlace component is to store window's
          positions & state.  Place calls to the component's Read method to set
          the Form's placement in either the Form's OnCreate or OnShow event
          handers.  Call the Write method on the Form's OnDestroy or OnClose
          method to save the Form's placement to the .INI file.

How to use:

       1) Choose Options|Install Components...|Add... and choose this file
          (the WINPLACE.DCR file must also reside in the same directory).  The
          TWinPlace component will be placed on the System page of the
          component palette.  [FYI: The .DCR file contains the component's
          bitmap image as shown on the component toolbar.  Use the Image Editor
          to change it.]

       2) Drop the TWinPlace component onto the desired form.  Use the Object
          Inspector to specify which .INI file to use (Windows will use your
          windows directory if none is specified; remember to include the .INI
          extension).  The Section property defaults to the Form's Name
          property, but you can change it to whatever you want (You may want
          several windows to use the same placement).  The Identifier property
          also defaults to the Form's name, but you can change it to whatever
          you want.  As an example, the results in one of my .INI files look
          like this, in the StartUp section, with three windows:
            [StartUp]
            MainWindow=0 1 -1 -1 -1 -1 713 409 988 723
            EventWindow=0 1 -1 -1 -1 -1 269 49 951 406
            ConfigurationWindow=0 1 -1 -1 -1 -1 197 412 705 710

       3) Call the WinPlace component's Read method to read the .INI file and
          set the Form's placement (usually in the Form's OnCreate or OnShow
          event handler).

                procedure TMyForm.FormCreate(Sender: TObject);
                begin
                  WinPlace1.Read;
                end;

          Use the OnCreate event handler if the Form is to be shown immediately,
          or use the OnShow event if the Form is to be created but not shown
          until a later time (the Read method will set the Form's Visible
          property to its state when last saved; ie when you last called the
          Write method).

       4) Call the WinPlace component's Write method to save the Form's
          placement into the .INI file (usually in the Form's OnClose or
          OnDestroy event handler).

                procedure TMyForm.FormDestroy(Sender: TObject);
                begin
                  WinPlace1.Write;
                end;

       5) Test!  The Form will be restored to its saved window state (maximized,
          minimized, hidden, etc.).  It will even remember it's normal position
          and its iconized position (note: if the Form is the Main Form of an
          application, it will not be restored in a minimized state).

       6) References:  The only "magic" in all of this is understanding the
          "window placement" routines within Delphi.  Check out Delphi's help
          under the following topics: TWindowPlacement, SetWindowPlacement, and
          GetWindowPlacement.  All of the rest is "mechanics".

          I/phil originally found the original version on the Delphi CompuServe
          Forum ("The greatest aid there is in learning Delphi"/prh).  I've
          never met Barry Harkness, nor have tried to contact him; but I sure
          appreciate his "style".  If you wish to send comments, revisions, etc.
          to me, I am Phil Henningsen, phil@surf.com.
*)

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs;

type
  TWinPlace = class(TComponent)
  private
    { Private declarations }
    FIniFileName: string;
    FSection: string;
    FIdentifier: string;
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner : TComponent); override;
    procedure Write; virtual;
    procedure Read; virtual;
  published
    { Published declarations }
    property IniFileName: string read FIniFileName write FIniFileName;
    property Section: string read FSection write FSection;
    property Identifier: string read FIdentifier write FIdentifier;
  end;

procedure Register;

{ ---------------------------------------------------------------------------- }

implementation

uses IniFiles;

constructor TWinPlace.Create(AOwner: TComponent);

begin
  inherited Create(AOwner);
  if Section = '' then
    Section :=AOwner.Name;
  if Identifier = '' then
    Identifier := AOwner.Name;
end;

{ ---------------------------------------------------------------------------- }

function GetNextInteger(var AStr: string; Default: integer): integer;
var
  BStr: string;
  Posn: integer;
begin
  Posn := Pos(' ', AStr);
  if Posn > 0 then
    begin
    BStr   := Copy(AStr, 1, Posn-1);
    AStr   := Copy(Astr, Posn+1, 255);
    Result := StrToInt(BStr);
    end
  else
    begin
    Result := Default;
    end;
end;

procedure TWinPlace.Read;

var
  IniFile: TIniFile;
  AWinPlace: TWindowPlacement;
  TmpStr: string;
  Posn: integer;

begin
  if IniFileName = '' then
    Exit;
  IniFile :=TIniFile.Create(IniFileName);
  try
    AWinPlace.length :=SizeOf(TWindowPlacement);
    with AWinPlace do
      begin
      TmpStr                  := IniFile.ReadString(Section, Identifier, '') + ' ';
      Flags                   := GetNextInteger(TmpStr, 0);
      ShowCmd                 := GetNextInteger(TmpStr, sw_ShowNormal);
      ptMinPosition.X         := GetNextInteger(TmpStr, 0);
      ptMinPosition.Y         := GetNextInteger(TmpStr, 0);
      ptMaxPosition.X         := GetNextInteger(TmpStr, 0);
      ptMaxPosition.Y         := GetNextInteger(TmpStr, 0);
      rcNormalPosition.Left   := GetNextInteger(TmpStr, 0);
      rcNormalPosition.Top    := GetNextInteger(TmpStr, 0);
      rcNormalPosition.Right  := GetNextInteger(TmpStr, 0);
      rcNormalPosition.Bottom := GetNextInteger(TmpStr, 0);
      if rcNormalPosition.Right > rcNormalPosition.Left then
        SetWindowPlacement(TForm(Owner).Handle, @AWinPlace)
      end;
  finally
    IniFile.Free;
  end;
end;

{ ---------------------------------------------------------------------------- }

function PutNextInteger(AStr: string; AInt: integer): string;
begin
  Result := AStr + IntToStr(AInt) + ' ';
end;

procedure TWinPlace.Write;

var
  IniFile: TIniFile;
  AWinPlace: TWindowPlacement;
  TmpStr: string;

begin
  if IniFileName = '' then
    Exit;
  AWinPlace.Length :=SizeOf(TWindowPlacement);
  if not GetWindowPlacement(TForm(Owner).Handle, @AWinPlace) then
    begin
    raise EAbort.Create('TWinPlace: Unable to retrieve window placement');
    Exit;
    end;
  IniFile :=TIniFile.Create(IniFileName);
  TmpStr  := '';
  try
    with AWinPlace do
      begin
      TmpStr := PutNextInteger(TmpStr, Flags);
      TmpStr := PutNextInteger(TmpStr, ShowCmd);
      TmpStr := PutNextInteger(TmpStr, ptMinPosition.X);
      TmpStr := PutNextInteger(TmpStr, ptMinPosition.Y);
      TmpStr := PutNextInteger(TmpStr, ptMaxPosition.X);
      TmpStr := PutNextInteger(TmpStr, ptMaxPosition.Y);
      TmpStr := PutNextInteger(TmpStr, rcNormalPosition.Left);
      TmpStr := PutNextInteger(TmpStr, rcNormalPosition.Top);
      TmpStr := PutNextInteger(TmpStr, rcNormalPosition.Right);
      TmpStr := PutNextInteger(TmpStr, rcNormalPosition.Bottom);
      IniFile.WriteString(Section, Identifier, TmpStr);
      end;
  finally
    IniFile.Free;
  end;
end;

{ ---------------------------------------------------------------------------- }

procedure Register;
begin
  RegisterComponents('System', [TWinPlace]);
end;

end.
